/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/

#include <iostream>
#include <cufft.h>
#include <cuda.h>
#include <cuda_runtime.h>
#include <curand.h>
#include <sstream>

#include "RawHdf5_hdf5.h"

///Initialised the signal of the data with a random normal distribution
/**	@param[out] data : Data to be initialised
*/
void initialisationSignalWithRandomNormal(RawHdf5 & data){
	int nbEvent = data.getNbEntries();
	std::cout << "initialisationSignalWithRandomNormal : allocate signal on GPU with "<<nbEvent<<" events" << std::endl;
	float *device_signal = NULL;
	cudaMalloc((void**)&device_signal, sizeof(float)*RAWHDF5_SIGNAL_0*nbEvent);	//Allocate data on device (only for signal)
	std::cout << "initialisationSignalWithRandomNormal : initlialise the CUDA random generator" << std::endl;
	curandGenerator_t generator;
	curandCreateGenerator(&generator, CURAND_RNG_PSEUDO_DEFAULT);			//Create the generator
	curandSetPseudoRandomGeneratorSeed(generator, 42);				//Set the random seed
	std::cout << "initialisationSignalWithRandomNormal : generate random numbers" << std::endl;
	curandGenerateNormal(generator, (float*)device_signal, RAWHDF5_SIGNAL_0*nbEvent, 2.0f, 1.0f);	//Generate a normal distribution (generator, data, size, mean, std)
	curandDestroyGenerator(generator);						//Destroy the generator
	
	std::cout << "initialisationSignalWithRandomNormal : getting back data from the GPU" << std::endl;
	cudaMemcpy(data.getSignalFull(), device_signal, sizeof(float)*RAWHDF5_SIGNAL_0*nbEvent, cudaMemcpyDeviceToHost);	//Copy data from GPU to Host
	
	cudaFree(device_signal);
}

///Create a HDF5 file with a Random Normal distribution
/**	@param baseOutputFileName : base name of the output file
 * 	@param nbEvent : number of event to be generated
*/
void createRandomNormalFile(const std::string & baseOutputFileName, int nbEvent){
	RawHdf5H5 hfile;
	hfile.resize(nbEvent);
	
// 	RawHdf5 data;
// 	hfile.clone(data);	//Let's link the file data without copying it
	
	initialisationSignalWithRandomNormal(hfile);
	
	std::stringstream fileName;
	fileName << baseOutputFileName << "_" << nbEvent << ".h5";
	
	hfile.write(fileName.str());
	std::cout << "Done" << std::endl;
}

int main(int argc, char** argv){
	int nbEvent = 2048000;
	if(argc < 2 || argc > 4){
		std::cout << "Usage :" << std::endl;
		std::cout << "\tcreate_hdf5_normal_distribution output" << std::endl;
		std::cout << "\tcreate_hdf5_normal_distribution output 2048000" << std::endl;
		std::cout << "Parameters :" << std::endl;
		std::cout << "\tHDF5 output file to be created" << std::endl;
		std::cout << "\tNumber of events to be generated" << std::endl;
		return 0;
	}
	std::string baseOutputFileName(argv[1]);
	if(argc >= 3){
		nbEvent = atoi(argv[2]);
	}
	createRandomNormalFile(baseOutputFileName, nbEvent);
	return 0;
}

